/*
 * Copyright 2002  John W. De Boskey <jwd@FreeBSD.Org>
 * All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JOHN W. DE BOSKEY ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL JOHN W. DE BOSKEY BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */

#include <machine/reg.h>
#include <sys/types.h>
#include <sys/ptrace.h>

#include "ptrace.h"

#define eflag_TF 0x100      /* TraceFlag */

unsigned int
get_long(pid, addr)
pid_t pid;
caddr_t addr;
{
  int rc;

  rc = -2;

  rc = ptrace ( PT_READ_I, pid, addr, 0 );

  return rc;
}

void
get_bytes(pid, addr, buffer, length)
pid_t   pid;
caddr_t addr;
caddr_t buffer;
int     length;
{
   int i;

   for (i=0; i<length; i+=4)
      *(unsigned int *)&buffer[i] = get_long(pid,addr+i);
   
   return;
}

void
get_string(pid, addr, buffer, length)
pid_t   pid;
caddr_t addr;
caddr_t buffer;
int     length;
{
   int i;

   for (i=0; i<length; i+=4) {
      *(unsigned int *)&buffer[i] = get_long(pid,addr+i);
      /*
       * Did we transfer a null terminator?
       */
      if ( *(char *)&buffer[i]   == 0 ||
           *(char *)&buffer[i+1] == 0 ||
           *(char *)&buffer[i+2] == 0 ||
           *(char *)&buffer[i+3] == 0)
         return;
   }
   return;
}

int
get_regs(pid,regs)
pid_t pid;
struct reg *regs;
{
   int rc;

   rc = ptrace(PT_GETREGS,pid,(caddr_t)regs,0);

   return(rc);
}

int
set_regs(pid,regs)
pid_t pid;
struct reg *regs;
{
   int rc;

   rc = ptrace(PT_SETREGS,pid,(caddr_t)regs,0);

   return(rc);
}

unsigned int
GET_SP(pid_t pid)
{
   struct reg regs;

   get_regs(pid,&regs);

   return(regs.r_esp);
}

unsigned int
GET_PC(pid_t pid)
{
   struct reg regs;

   get_regs(pid,&regs);

   return(regs.r_eip);
}

void
set_singlestep(pid)
pid_t pid;
{
   struct reg regs;

   get_regs(pid,&regs);

   regs.r_eflags |= eflag_TF;

   set_regs(pid,&regs);

   return;
}

